#!/usr/bin/perl

package HTFeed::PackageType::Google::METS;
use strict;
use warnings;
use Log::Log4perl qw(get_logger);
use base qw(HTFeed::METS);

sub _add_header {
    my $self = shift;
    $self->SUPER::_add_header();
    my $volume = $self->{volume};
    my $xpc = $volume->get_source_mets_xpc();

    # Google: altRecordID handling - reject if there is an altRecordID in the
    # source METS. This should only happen if the volume is a duplicate, which
    # should be detected by looking for condition 31 set and source library
    # bibkey not null, but it doesn't hurt to check.


    foreach my $alt_record_id ($xpc->findnodes('//mets:altRecordID')) {
        $self->set_error("BadFile",file => $volume->get_source_mets_file(),
            field => 'altRecordID',
            value=>$alt_record_id->toString(),
            detail=>'altRecordID found in Google METS');
    }
}

sub _add_source_mets_events {
    my $self = shift;
    my $volume = $self->{volume};
    my $premis = $self->{premis};
    my $xpc = $volume->get_source_mets_xpc();

    my %google_event_map = (
        'Scan' => 'capture',
        ### TODO: review following four events
        'Analyze' => 'image analysis',
        'Process' => 'image processing', 
        'Audit' => 'manual quality review',
        'Rubbish' => 'partial manual quality review',
    );

    foreach my $event ($xpc->findnodes('//premis:event')) {
        my $google_type = $xpc->findvalue('./premis:eventType',$event);
        if(my $type = $google_event_map{$google_type}) {
            my $eventconfig = {
                type => $type,
                date => $xpc->findvalue('./premis:eventDateTime',$event),
                detail =>  $xpc->findvalue('./premis:eventDetail',$event),
            };
            # Strip off the seconds so that we can reliably match events with ones
            # previously listed via GRIN
            $eventconfig->{date} =~ s/:\d{2}$/:00/;
            $eventconfig->{'eventid'} =  $volume->make_premis_uuid($type,$eventconfig->{'date'});

            my @agents = $xpc->findnodes('./premis:linkingAgentIdentifier',$event);
            if(@agents != 1) {
                $self->set_error("BadField",field => "linkingAgentIdentifier",
                    detail => "Expected 1 linking agent, found " . scalar(@agents));
            }
            my $agent = $agents[0];
            my $agentid = $xpc->findvalue('./premis:linkingAgentIdentifierValue',$agent);
            if($agentid eq 'Google (Firm)') {
                $eventconfig->{'executor_type'} = 'MARC21 Code',
                $eventconfig->{'executor'} = 'Ca-MvGOO',
            } else {
                $self->set_error("BadField",field=>"linkingAgentIdentifierValue",
                    expected => "Google, Inc. (Firm)", actual => $agentid, 
                    detail => "Unknown agent ID");
            }

            $self->add_premis_event($eventconfig);
        } elsif($google_type eq 'Convert') {
            foreach my $eventcode (('source_mets_creation','page_md5_create','image_compression')) {
                my $eventconfig = $volume->get_nspkg()->get_event_configuration($eventcode);
                $eventconfig->{'date'} = $xpc->findvalue('./premis:eventDateTime',$event);
                $eventconfig->{'date'} =~ s/:\d{2}$/:00/;
                $eventconfig->{'executor_type'} = 'MARC21 Code';
                $eventconfig->{'executor'} = 'Ca-MvGOO';
                # override tools from config - Google doesn't list any, and the HT tools
                # are not applicable here, just the description
                delete $eventconfig->{'tools'};
                $eventconfig->{'eventid'} = $volume->make_premis_uuid($eventconfig->{'type'},$eventconfig->{'date'});
                $self->add_premis_event($eventconfig);
            }

        }

    }
}

sub _add_techmds {
    my $self = shift;
    my $volume = $self->{volume};
    my $xc = $volume->get_source_mets_xpc();
    $self->SUPER::_add_techmds();

    my $reading_order = new METS::MetadataSection( 'techMD',
        id => $self->_get_subsec_id('TMD'));

    my @mdwraps = $xc->findnodes('//METS:mdWrap[@LABEL="reading order"]');
    if(@mdwraps != 1) {
        my $count = scalar(@mdwraps);
        $self->set_error("BadField",field=>"reading order",decription=>"Found $count reading order techMDs, expected 1");
    }
    $reading_order->set_mdwrap($mdwraps[0]);
    push(@{ $self->{amd_mdsecs} },$reading_order);
}


sub _extract_old_premis {
    my $self = shift;
    my $volume = $self->{volume};
    my $nspkg = $volume->get_nspkg();
    my $xpc = $volume->get_repository_mets_xpc();
    return unless defined $xpc;

    my %premis1_event_map = (
        capture => [qw(capture)],
        # we know the Google source METS & image compression was done at the same time as page md5 creation
        'message digest calculation' => [qw(page_md5_create source_mets_creation image_compression)],
        decryption => [qw(decryption)],
        'fixity check' => [qw(page_md5_fixity)],
        'ingestion' => [qw(ingestion zip_compression zip_md5_create)],
        'validation' => [qw(package_validation)]
    );

    foreach my $event ($xpc->findnodes('//premis1:event')) {
        my $premis1_type = $xpc->findvalue('./premis1:eventType',$event);
        if(my $eventcodes = $premis1_event_map{$premis1_type}) {
            foreach my $eventcode (@$eventcodes) {
                my $eventconfig = $nspkg->get_event_configuration($eventcode);
                $eventconfig->{date} = $xpc->findvalue('./premis1:eventDateTime',$event);
                $eventconfig->{'eventid'} =  $volume->make_premis_uuid($eventconfig->{'type'},$eventconfig->{'date'});
                # tool is not represented in PREMIS1; don't make up one
                delete $eventconfig->{'tools'};

                my @agents = $xpc->findnodes('./premis1:linkingAgentIdentifier',$event);
                if(@agents != 1) {
                    $self->set_error("BadField",field => "linkingAgentIdentifier",
                        detail => "Expected 1 linking agent, found " . scalar(@agents));
                }
                my $agent = $agents[0];
                my $agentid = $xpc->findvalue('./premis1:linkingAgentIdentifierValue',$agent);
                if($agentid eq 'Google, Inc.') {
                    $eventconfig->{'executor_type'} = 'MARC21 Code';
                    $eventconfig->{'executor'} = 'Ca-MvGOO';
                } elsif($agentid eq 'UM') {
                    $eventconfig->{'executor_type'} = 'MARC21 Code';
                    $eventconfig->{'executor'} = 'MiU';
                } else {
                    $self->set_error("BadField",field=>"linkingAgentIdentifierValue",
                        actual => $agentid, 
                        detail => "Unknown agent ID");
                }

                $self->add_premis_event($eventconfig);
            }
        }
    }

    # get any PREMIS2 events if they are there..
    return $self->SUPER::_extract_old_premis();
}

1;
