#!/usr/bin/perl

package HTFeed::PackageType::Google::METS_PREMIS1;
use strict;
use warnings;
use Log::Log4perl qw(get_logger);
use HTFeed::XMLNamespaces qw(:namespaces :schemas);
use HTFeed::METS;
use base qw(HTFeed::METS);

sub _add_header {
    my $self = shift;
    $self->SUPER::_add_header();
    my $volume = $self->{volume};
    my $xpc = $volume->get_source_mets_xpc();

    # Google: altRecordID handling - reject if there is an altRecordID in the
    # source METS. This should only happen if the volume is a duplicate, which
    # should be detected by looking for condition 31 set and source library
    # bibkey not null, but it doesn't hurt to check.


    foreach my $alt_record_id ($xpc->findnodes('//mets:altRecordID')) {
        $self->set_error("BadFile",file => $volume->get_source_mets_file(),
            field => 'altRecordID',
            value=>$alt_record_id->toString(),
            detail=>'altRecordID found in Google METS');
    }
}

sub _add_schemas {
    my $self = shift;
    my $mets = $self->{mets};

    # add PREMIS1 namespace but don't worry about validating it
    $mets->add_schema( "PREMIS", NS_PREMIS1);

}

sub _get_source_mets_events {
    my $self = shift;
    my $volume = $self->{volume};
    my $premis = $self->{premis};
    my $xpc = $volume->get_source_mets_xpc();

    my $srcmets_event_dates = {};


    foreach my $event ($xpc->findnodes('//premis:event')) {
        my $google_type = $xpc->findvalue('./premis:eventType',$event);
        my $google_date = $xpc->findvalue('./premis:eventDateTime',$event);

        # Strip off the seconds so that we can reliably match events with ones
        # previously listed via GRIN
        $google_date =~ s/:\d{2}$/:00/;
        $srcmets_event_dates->{$google_type} = $google_date;

    }

    return $srcmets_event_dates;
}

sub _add_premis {
    my $self = shift;
    my $volume = $self->{volume};
    my $nspkg = $volume->get_nspkg();
    my $xpc = $volume->get_repository_mets_xpc();
    $self->{parser} = new XML::LibXML;

    # preservationLevel

    # keep track of count for each kind of event
    $self->{premis_xml} = "";
    $self->{event_type_counters} = {};
    $self->{old_events} = {};

    my $premis_obj_MD =
      new METS::MetadataSection( 'techMD', 'id' => 'premisobject1' );
    $premis_obj_MD->set_xml_node( $self->{parser}->parse_xml_chunk(<<"EOT"), mdtype => 'PREMIS' );

        <PREMIS:object xmlns:PREMIS="http://www.loc.gov/standards/premis">
               <PREMIS:preservationLevel>1</PREMIS:preservationLevel>
        </PREMIS:object>
EOT
    push( @{ $self->{amd_mdsecs} }, $premis_obj_MD );

    $self->{premis_node} = new XML::LibXML::DocumentFragment();

    # old PREMIS events
    if(defined $xpc) {
        foreach my $event ($xpc->findnodes('//premis1:event')) {
            my $PREMIS_id = $xpc->findvalue('.//premis1:eventIdentifierValue',$event);
            my $PREMIS_type = $xpc->findvalue('.//premis1:eventType',$event);
            my $PREMIS_date = $xpc->findvalue('.//premis1:eventDateTime',$event);

            $self->{old_events}{$PREMIS_type}{$PREMIS_date} = $event;

            # canonicalize to ensure namespaces handled correctly
            $self->{premis_node}->appendChild($event);
            my ($eventIdPrefix,$eventCount) = $PREMIS_id =~ /^(\D+)(\d+)$/;
            if(defined $eventCount) {
                if(not defined $self->{event_type_counters}{$eventIdPrefix}
                        or $self->{event_type_counters}{$eventIdPrefix} < $eventCount) {
                    $self->{event_type_counters}{$eventIdPrefix} = $eventCount;
                }
            } else {
                $self->set_error("BadField",field => "premis1:eventIdentifierValue",actual=>$PREMIS_id);
            }
        }
    }

    # new events

    my $srcmets_event_date = $self->_get_source_mets_events();

    my $capture_agent = $volume->is_digifeed() ? 'UNKNOWN' : 'Google, Inc.';

    $self->_add_premis_event(type => 'capture', time => $srcmets_event_date->{Scan}, agent => $capture_agent);

    # compression -> Google, convert date from source METS
    $self->_add_premis_event(type => 'compression', time => $srcmets_event_date->{Convert}, agent => 'Google, Inc.');

    # message digest calculation -> Google, convert date from source METS
    $self->_add_premis_event(type => 'message digest calculation', 
        time => $srcmets_event_date->{Convert}, agent => 'Google, Inc.');

    $self->_add_premis_event(type => 'decryption', time => $self->_get_event_time('decryption'), agent => 'UM');

    $self->_add_premis_event(type => 'fixity check', time => $self->_get_event_time('page_md5_fixity'), agent => 'UM',
                           outcome => <<"EOT");
             <PREMIS:eventOutcomeInformation>
                    <PREMIS:eventOutcomeDetail>pass</PREMIS:eventOutcomeDetail>
             </PREMIS:eventOutcomeInformation>
EOT

    $self->_add_premis_event(type => 'validation', time => $self->_get_event_time('package_validation'), agent => 'UM');

    # ingestion -> now
    # last chance to record, even though it's not done yet
    $volume->record_premis_event('ingestion');
    $self->_add_premis_event(type => 'ingestion', time => $self->_get_event_time('ingestion'), agent => 'UM');

    my $digiprovMD =
      new METS::MetadataSection( 'digiprovMD', 'id' => 'premisevent1' );
    $digiprovMD->set_xml_node($self->{premis_node}, mdtype => 'PREMIS' );
    push( @{ $self->{amd_mdsecs} }, $digiprovMD );

    return;
}

sub _add_premis_event {
    my $self = shift;
    my %params = @_;
    my $namespace = NS_PREMIS1;

    foreach my $param (qw(type agent time)) {
        if(not defined $params{$param} or $params{$param} eq '') {
            $self->set_error("MissingField",field=>"premis_$param",actual=>\%params);
        }
    }

    my $outcome = "";
    $outcome = "\n" . $params{outcome} if defined $params{outcome};

    # Haven't seen this event: get a new event ID and add it
    if(not defined ($self->{'old_events'}{$params{'type'}}{$params{'time'}})) {
    

        my $eventSeq = ++$self->{event_type_counters}{$params{type}};

        $self->{premis_node}->appendChild( $self->{parser}->parse_xml_chunk(<<EOT));
         <PREMIS:event xmlns:PREMIS="$namespace">
          <PREMIS:eventIdentifier>
           <PREMIS:eventIdentifierValue>$params{type}$eventSeq</PREMIS:eventIdentifierValue>
          </PREMIS:eventIdentifier>
          <PREMIS:eventType>$params{type}</PREMIS:eventType>
          <PREMIS:eventDateTime>$params{time}</PREMIS:eventDateTime>$outcome
          <PREMIS:linkingAgentIdentifier>
           <PREMIS:linkingAgentIdentifierType>AgentID</PREMIS:linkingAgentIdentifierType>
           <PREMIS:linkingAgentIdentifierValue>$params{agent}</PREMIS:linkingAgentIdentifierValue>
          </PREMIS:linkingAgentIdentifier>
         </PREMIS:event>
EOT
    }

}

# get just the event time from the recorded premis event, since that's all we need for
# PREMIS 1
sub _get_event_time {
    my $self = shift;
    my $event_id = shift;

    my ($eventid, $date, $outcome_node) = $self->{volume}->get_event_info($event_id);

    return $date;

}

# Override parent class: don't add source METS; add content filegroups in specific order
sub _add_filesecs {
    my $self = shift;

    $self->_add_zip_fg();
    my $mets   = $self->{mets};
    my $volume = $self->{volume};

    # then add the actual content files
    my $filegroups = $volume->get_file_groups();
    $self->{filegroups} = {};

    foreach my $filegroup_name (qw(image ocr hocr)) {
        my $filegroup = $filegroups->{$filegroup_name};
        my $mets_filegroup = new METS::FileGroup(
            id  => $self->_get_subsec_id("FG"),
            use => $filegroup->get_use()
        );
        $mets_filegroup->add_files( $filegroup->get_filenames(),
            prefix => $filegroup->get_prefix(),
            path => $volume->get_staging_directory() );

        $self->{filegroups}{$filegroup_name} = $mets_filegroup;
        $mets->add_filegroup($mets_filegroup);
    }

}

1;
